@@ -37,21 +37,23 @@ MclustDA <- function(data, class, G = NULL, modelNames = NULL,
     stop("G must be positive")
   #
   if(is.null(modelNames)) 
-    { if(oneD) modelNames <- c("E", "V")
-      else     modelNames <- mclust.options("emModelNames")
-  }
-  if(n <= p) 
-    { m <- match(c("EEE","EEV","VEV","VVV"), 
-                 mclust.options("emModelNames"), nomatch=0)
-      modelNames <- modelNames[-m]
+  { 
+    if(oneD) modelNames <- c("E", "V")
+    else     modelNames <- mclust.options("emModelNames")
   }
   if(!is.list(modelNames))
     { modelNames <- rep(list(modelNames), ncl) }
-  #
-  # hcUse <- mclust.options("hcUse")
-  # mclust.options("hcUse" = "VARS")
-  # on.exit(mclust.options("hcUse" = hcUse))
-  #
+  for(k in 1:ncl)
+  {
+    if(n*prop[k] <= p)
+    {
+      m <- match(modelNames[[k]], 
+                 c("EII", "VII", "EEI", "VEI", "EVI", "VVI"),
+                 nomatch = 0)
+      modelNames[[k]] <- modelNames[[k]][m]
+    }
+  }
+
   if(modelType == "EDDA")
   { 
     mc[[1]] <- as.name("mstep")
@@ -83,12 +85,11 @@ MclustDA <- function(data, class, G = NULL, modelNames = NULL,
       { warning("No model(s) can be estimated!!")
         return() }
     names(BIC) <- modelNames
-    bic <- max(BIC, na.rm = TRUE)
-    loglik <- Model$loglik
-    df <- (2*loglik - bic)/log(Model$n)
-    # there are (nclass-1) more df than really needed
-    # equal to logLik(object) but faster
-    Model <- c(Model, list("BIC" = BIC))
+    # bic <- max(BIC, na.rm = TRUE)
+    # loglik <- Model$loglik
+    # df <- (2*loglik - bic)/log(Model$n)
+    # Model <- c(Model, list("BIC" = BIC))
+    bic <- loglik <- df <- NULL
     Models <- rep(list(Model), ncl)
     names(Models) <- classLabel
     for(l in 1:ncl)
@@ -137,11 +138,14 @@ MclustDA <- function(data, class, G = NULL, modelNames = NULL,
     Models <- rep(list(NULL), ncl)
     mc[[1]] <- as.name("mclustBIC")
     mc$class <- NULL
+    # noise <- eval(mc$initialization$noise, parent.frame())
     for(l in 1:ncl) 
        { I <- (class == classLabel[l])
          mc[[2]] <- data[I,]
          mc$G <- G[[l]]
          mc$modelNames <- as.character(modelNames[[l]])
+         # if(!is.null(noise))
+         #   mc$initialization$noise <- noise[I]
          if(verbose) cat(paste0("Class ", classLabel[l], ": "))
          BIC <- eval(mc, parent.frame())
          # slightly adjust parameters if none of the models can be fitted
@@ -162,22 +166,19 @@ MclustDA <- function(data, class, G = NULL, modelNames = NULL,
     }
     bic <- loglik <- df <- NULL
   }
-  
-  names(Models) <- classLabel
   Models$Vinv <- NULL
+  names(Models) <- classLabel
+  
   out <- list(call = call, data = data, class = class,
               type = modelType, n = n, d = p, prop = prop, 
               models = Models, bic = bic, loglik = loglik, df = df)
-  out <- structure(out, prior = prior, control = control, 
+  out <- structure(out, prior = prior, 
+                   control = control, 
                    class = "MclustDA")
-  
-  if(modelType == "MclustDA") 
-  { 
-    l <- logLik.MclustDA(out, data)
-    out$loglik <- as.numeric(l)
-    out$df <- attr(l, "df")
-    out$bic <- 2*out$loglik - log(n)*out$df
-  }
+  l <- logLik.MclustDA(out, data)
+  out$loglik <- as.numeric(l)
+  out$df <- attr(l, "df")
+  out$bic <- 2*out$loglik - log(n)*out$df
   
   return(out)
 }
@@ -342,10 +343,12 @@ getParameters.MclustDA <- function(object)
   # prior <- attr(object, "prior")
   par <- vector(mode = "list", length = nclass)
   for(i in seq(nclass))
-  { par[[i]] <- models[[i]]$parameters
+  { 
+		par[[i]] <- models[[i]]$parameters
     if(is.null(par[[i]]$pro)) par$pro <- 1
     if(par[[i]]$variance$d < 2)
-    { sigma <- rep(par[[i]]$variance$sigma,
+    { 
+			sigma <- rep(par[[i]]$variance$sigma,
                    models[[i]]$G)[1:models[[i]]$G]
       names(sigma) <- names(par[[i]]$mean)
       par[[i]]$variance$sigma <- sigma
@@ -362,29 +365,32 @@ logLik.MclustDA <- function (object, data, ...)
   n <- object$n
   d <- object$d
   par <- getParameters.MclustDA(object)
-  nclass <- length(par)
-  fclass <- sapply(object$models, function(m) m$n)/n
-  logfclass <- log(fclass)
-  G <- sapply(par, function(x) length(x$pro))
-  if(object$type == "EDDA") 
-    { df <- d * nclass + nVarParams(object$models[[1]]$modelName, 
-                                    d = d, G = nclass)
+  prop <- object$prop
+  nclass <- length(prop)
+  df <- if(object$type == "EDDA") 
+  { 
+    d * nclass + nVarParams(object$models[[1]]$modelName, 
+                                  d = d, G = nclass)
+  } else 
+  { 
+    sum(sapply(object$models, function(mod) 
+        with(mod, (G - 1) + G * d + nVarParams(modelName, d = d, G = G))))
   }
-  else 
-    { df <- sum(sapply(object$models, function(mod) with(mod, 
-                       (G - 1) + G * d + nVarParams(modelName, d = d, G = G))))
-  }
-  # ll <- sapply(object$models, function(mod) 
-  #       { do.call("dens", c(list(data = data, logarithm = FALSE), mod)) })
-  # l <- sum(log(apply(ll, 1, function(l) sum(fclass*l))))
+  df <- df + nclass-1
   ll <- sapply(object$models, function(mod) 
         { do.call("dens", c(list(data = data, logarithm = TRUE), mod)) })
-  l <- sum(apply(ll, 1, function(l) logsumexp(logfclass+l)))
-    
-  attr(l, "nobs") <- n
-  attr(l, "df") <- df
-  class(l) <- "logLik"
-  return(l)
+  
+  # logcden <- cdens(data = data,
+  #                  modelName = object$models[[1]]$modelName,
+  #                  parameters = object$models[[1]]$parameters,
+  #                  logarithm = TRUE)
+  
+  
+  ll <- sum(logsumexp(ll, log(prop)))
+  attr(ll, "nobs") <- n
+  attr(ll, "df") <- df
+  class(ll) <- "logLik"
+  return(ll)
 }
 
 predict.MclustDA <- function(object, newdata, prop = object$prop, ...)
@@ -422,9 +428,11 @@ predict.MclustDA <- function(object, newdata, prop = object$prop, ...)
   z <- matrix(as.double(NA), nrow = NROW(newdata), ncol = nclass)
   for(j in 1:nclass)
     z[,j] <- densfun(models[[j]], data = newdata)
-  z <- sweep(z, MARGIN = 2, FUN = "+", STATS = log(prop))
-  z <- sweep(z, MARGIN = 1, FUN = "-", STATS = apply(z, 1, logsumexp))
-  z <- exp(z)
+  # TODO: to be removed at a certain point
+  # z <- sweep(z, MARGIN = 2, FUN = "+", STATS = log(prop))
+  # z <- sweep(z, MARGIN = 1, FUN = "-", STATS = apply(z, 1, logsumexp_old))
+  # z <- exp(z)
+  z <- softmax(z, log(prop))
   colnames(z) <- classNames
   cl <- apply(z, 1, which.max)
   class <- factor(classNames[cl], levels = classNames)
@@ -901,85 +909,8 @@ plot.MclustDA <- function(x, what = c("scatterplot", "classification", "train&te
   }
     
   invisible()
-  
 }
 
-# TODO: old version to be deleted at a certain point
-# cvMclustDA <- function(object, nfold = 10, 
-#                        metric = c("error", "brier"), 
-#                        prop = object$prop,
-#                        verbose = interactive(), ...) 
-# {
-# 
-#   call <- object$call
-#   nfold <- as.numeric(nfold)
-#   metric <- match.arg(metric, 
-#                       choices = eval(formals(cvMclustDA)$metric), 
-#                       several.ok = FALSE)
-#   #
-#   data <- object$data
-#   class <- as.factor(object$class)
-#   n <- length(class)
-#   G <- lapply(object$models, function(mod) mod$G)
-#   modelName <- lapply(object$models, function(mod) mod$modelName)
-#   #
-#   ce <- function(pred, class)
-#   {
-#     1 - sum(class == pred, na.rm = TRUE)/length(class)
-#   }
-#   #
-#   folds <- if(nfold == n) lapply(1:n, function(x) x)
-#            else           balancedFolds(class, nfolds = nfold)
-#   nfold <- length(folds)
-#   folds.size <- sapply(folds, length)
-#   #
-#   cvmetric <- rep(NA, nfold)
-#   cvclass <- factor(rep(NA, n), levels = levels(class))
-#   cvprob  <- matrix(as.double(NA), nrow = n, ncol = nlevels(class),
-#                     dimnames = list(NULL, levels(class)))
-#   
-#   if(verbose)
-#   { 
-#     cat("cross-validating ...\n")
-#     flush.console()
-#     pbar <- txtProgressBar(min = 0, max = nfold, style = 3)
-#     on.exit(close(pbar))
-#   }
-#   
-#   for(i in seq(nfold))
-#   { 
-#     x <- data[-folds[[i]],,drop=FALSE]
-#     y <- class[-folds[[i]]]
-#     call$data <- x
-#     call$class <- y
-#     call$G <- G
-#     call$modelNames <- modelName
-#     call$verbose <- FALSE
-#     mod <- eval(call, parent.frame())
-#     #
-#     predTest <- predict(mod, data[folds[[i]],,drop=FALSE], prop = prop)
-#     cvmetric[i] <- if(metric == "error") 
-#                      ce(predTest$classification, class[folds[[i]]])
-#                    else 
-#                      BrierScore(predTest$z, class[folds[[i]]])
-#     cvclass[folds[[i]]] <- predTest$classification
-#     cvprob[folds[[i]],] <- predTest$z
-#     #
-#     if(verbose) 
-#       setTxtProgressBar(pbar, i)
-#   }
-#   #    
-#   cv <- sum(cvmetric*folds.size)/sum(folds.size)
-#   se <- sqrt(var(cvmetric)/nfold)
-#   #
-#   out <- list(classification = cvclass, 
-#               z = cvprob,
-#               error = if(metric == "error") cv else NA,
-#               brier = if(metric == "brier") cv else NA,
-#               se = se)
-#   return(out)
-# }
-
 cvMclustDA <- function(object, nfold = 10, 
                        prop = object$prop,
                        verbose = interactive(), 
